animation_data = {}

local mt = {__index = animation_data}

function animation_data.new(image, w, h, desc)
	local a = {}
	setmetatable(a, mt)
	a:initialize(image, w, h, desc)
	return a
end

function animation_data:initialize(image, frame_w, frame_h, desc)
	self.modes = self.modes or {}
	self.img_name = image[2]
	self.img = image[1]
	self.frame_w = frame_w
	self.frame_h = frame_h
	self.desc = desc

	local imgh = self.img:getHeight()
	local imgw = self.img:getWidth()

	local i = 0
	
	for k, v in ipairs(desc) do
		self.modes[v.name] = self.modes[v.name] or {}

		local mode = self.modes[v.name]
		
		mode.offsets = v.offsets

		mode.frame_count = v.frame_count
		mode.frames = {}
		mode.fps = v.fps or 0
		for a = 1, mode.frame_count do
			local start_x = (i * frame_w) % imgw
			local start_y = math.floor(i * frame_w / imgw) * frame_h
			mode.frames[a] = love.graphics.newQuad(start_x, start_y, 
				frame_w, frame_h, imgw, imgh)
			i = i + 1
		end
	end
end

function create_animation(image, frame_w, frame_h, desc)
	return animation_data:new(image, frame_w, frame_h, desc)
end

function animation_data:deserialize()
	self:initialize(get_image(self.img_name), self.frame_w, self.frame_h, self.desc)
end

animation = {}
local anim_mt = {__index = animation}

function animation.new(adata)
	local a = {}
	setmetatable(a, anim_mt)
	a:initialize(adata)
	return a
end

function animation:initialize(anim_data)
	self.anim_data = anim_data
	self.mode = nil
end

function animation:get_w()
	return self.anim_data.frame_w
end

function animation:get_h()
	return self.anim_data.frame_h
end

function animation:animate(dt)
	if self.mode == nil then return end

	self.time = self.time + dt

	if self.time > 1 / self.mode.fps then
		self.time = self.time - 1 / self.mode.fps
		self.frame = self.frame + 1
		if self.frame > self.mode.frame_count then
			if self.loop then
				self.frame = 1
			else
--				self.frame = self.mode.frame_count
				self.mode = nil
			end
		end
	end
end

function animation:set_mode(mode, loop)
	assert(self.anim_data.modes[mode], "mode \"" .. mode .. "\" doesn't exist")
	if self.loop and self.mode == self.anim_data.modes[mode] then
		return
	end
	self.mode = self.anim_data.modes[mode]
	self.frame = 1
	self.time = 0
	self.loop = loop or false
end

function animation:draw(x, y, sx, sy)
	if self.mode == nil then
		return
	end
	local x_pos = x
	local y_pos = y
	if self.mode.offsets then
		x_pos = x_pos + self.mode.offsets[self.frame].x
		y_pos = y_pos + self.mode.offsets[self.frame].y
	end
	love.graphics.drawq(self.anim_data.img, self.mode.frames[self.frame], x_pos, y_pos, 0, sx, sy)
end

