require 'misc'

local config = {}
local config_file_def = "config_default"

---
-- Read a config file.
-- The file should be in Lua and return a table.
-- The file should consist only of tables, strings, booleans, numbers and
-- shouldn't have cyclic references.
-- The loaded table will be merged with a one loaded from config_file_def.
--
-- @param file The config file.
function read_config(file)
	local success
	local file = file or "config"
	if love.filesystem.exists(file) then
		local chunk = love.filesystem.load(file)
		success, config = pcall(chunk)
		if not success then
			config = {}
			print("Couldn't load config file: ", config)
		else
			config = load_table(config)
		end
	end

	defaults = love.filesystem.load(config_file_def)()
	merge(config, defaults)
	write_config("config")
	return config
end

---
-- Write the config.
function write_config(file)
	local f = love.filesystem.newFile(file or "config")
	f:open("w")
	save_table(f, config)
	f:close()
	return ret
end

