require 'hump/vector'
require 'hump/camera'
require 'creature'
require 'timer'
require 'collision'
require 'shot'
require 'wave'
require 'hiscore'
require 'sounds'

game = {}
keysdown = {}

local bg = get_image_raw("bg2.png")
local shadow = get_image_raw("shadow.png")

GAME_W = 800
GAME_H = 600

PLAYER_SPEED = 300
SHOOT_DELAY = 0.2

local SCOREBOX = { pos = vector(0, 0), size = vector(340, 120), }
local LIVESBOX = { pos = vector(568, 0), size = vector(230, 120), }
local HEALTHBOX = { pos = vector(0, 0), size = vector(GAME_W, 24), }

function game:keypressed(key, uni)
	if key == "escape" then
		menu:loadmenu("menu_pause")
		Gamestate.switch(menu)
	end

	keysdown[key] = true
end

function game:keyreleased(key, uni)
	keysdown[key] = false
end

function game:mousepressed(x, y, button)
end

function game:mousereleased()
end

function game:restart_game()
	self:restart(self.start_lvl)
end

function game:restart(level)
	self.level = level or 0
	if level then
		self.start_lvl = level
	end

	self.player = {
		pos = vector(GAME_W/2, GAME_H/2),
		vel = vector(0, 0),
		size = vector(56, 68),
		lives = 3,
		dead = false,
		image = get_image_raw("frok.png"),
		last_shot = 0,
		points = 0
	}

	self:respawn()
end

function game:respawn()
	self.player.pos = vector(GAME_W/2, GAME_H/2)
	self.player.last_shot = 0
	self.player.dead = false
	self.player.death_anim = nil
	
	self.clowns = {}
	self.player_shots = {}
	self.enemy_shots = {}

	self.max_id = 0
	self.timer = timer.create()

	self.last_wave = 0
	self.wave_clear = true
end

function game:delete_clown(id, no_points)
	for i, v in ipairs(self.clowns) do
		if v.id == id then
			if not no_points then
				self.player.points = self.player.points + creature_types[v.type].points
				set_score(self.player.points)
			end
			table.remove(self.clowns, i)
			break
		end
	end
end

function game:delete_shot(id)
	self:delete_obj(self.player_shots, id)
	self:delete_obj(self.enemy_shots, id)
end

function game:get_id()
	self.max_id = self.max_id + 1
	return self.max_id
end

function game:die()
	set_score(self.player.points)
	write_score()
	menu:loadmenu("menu_game_over")
	Gamestate.switch(menu)
end

function game:create_clown(clown_type, loc_base, loc_max_offset)
	local SPAWN_DIST = creature_types[clown_type].speed

	local dir = (loc_base + math.random(0, loc_max_offset)) % 12
	if dir == 0 then dir = 12 end
	
	local pos = vector(0, 0)

	if dir == 1 or dir == 2 or dir == 3 then
		-- from above
		pos.y = - creature_types[clown_type].size.y - SPAWN_DIST
	elseif dir == 4 or dir == 12 then
		-- from upper l/r
		pos.y = math.random(0, GAME_H * 0.333 - creature_types[clown_type].size.y)
	elseif dir == 5 or dir == 11 then
		-- from mid l/r
		pos.y = math.random(GAME_H * 0.333, GAME_H * 0.667 - creature_types[clown_type].size.y)
	elseif dir == 6 or dir == 10 then
		-- from lower l/r
		pos.y = math.random(GAME_H * 0.667, GAME_H - creature_types[clown_type].size.y)
	elseif dir == 7 or dir == 8 or dir == 9 then
		-- from below
		pos.y = GAME_H + SPAWN_DIST
	end
	
	if dir == 1 or dir == 9 then
		-- from left t/b
		pos.x = math.random(0, GAME_W * 0.333 - creature_types[clown_type].size.x)
	elseif dir == 2 or dir == 8 then
		-- from mid t/b
		pos.x = math.random(GAME_W * 0.333, GAME_W * 0.667 - creature_types[clown_type].size.x)
	elseif dir == 3 or dir == 7 then
		-- from right t/b
		pos.x = math.random(GAME_W  * 0.667, GAME_W - creature_types[clown_type].size.x)
	elseif dir == 4 or dir == 5 or dir == 6 then
		-- from right
		pos.x = GAME_W + SPAWN_DIST
	elseif dir == 10 or dir == 11 or dir == 12 then
		-- from left
		pos.x = - creature_types[clown_type].size.x - SPAWN_DIST
	end

	self:add_clown(pos, clown_type)
end

function game:add_clown(pos, type)
	local clown = creature.new(pos, type)
	clown.id = self:get_id()
	table.insert(self.clowns, clown)
end

function game:add_shot(pos, dir, type, by_player)
	local shot = shot.new(pos, dir, type)
	shot.id = self:get_id()
	
	if by_player then
		table.insert(game.player_shots, shot)
	else
		table.insert(game.enemy_shots, shot)
	end
end

function game:enter()
	keysdown = {}

	love.graphics.setColorMode("replace")
end

function game:leave()
	set_score(self.player.points)
	write_score()
end

function game:draw()

	love.graphics.setColorMode("replace")

	love.graphics.draw(bg, 0, 0)
	
	if not self.player.dead then
		love.graphics.draw(shadow,
			self.player.pos.x + self.player.size.x/2 - 6*SCALE,
			self.player.pos.y + self.player.size.y - 4*SCALE,
			0, SCALE)
	end
	
	for i, v in ipairs(self.clowns) do
		love.graphics.draw(shadow,
			v.pos.x + v.size.x/2 - 6*SCALE,
			v.pos.y + v.size.y - 4*SCALE,
			0, SCALE)
	end

	if not self.player.dead then
		love.graphics.draw(self.player.image, 
			self.player.pos.x, 
			self.player.pos.y,
			0, SCALE)
	else
		self.player.death_anim:draw(
			math.ceil(self.player.pos.x) + self.player.size.x/2 - 32, 
			math.ceil(self.player.pos.y) + self.player.size.y/2 - 32)
	end
	
	
	table.sort(self.clowns, function(a, b)
		return a.pos.y + a.size.y < b.pos.y + b.size.y
	end)

	for i, v in ipairs(self.clowns) do
		v:draw()
	end

	for i, v in ipairs(self.player_shots) do
		v:draw()
	end

	for i, v in ipairs(self.enemy_shots) do
		v:draw()
	end

	love.graphics.setColorMode("modulate")

	love.graphics.setColor(255, 255, 255, self:calculate_alpha(LIVESBOX))
	love.graphics.printf(
		"Lives: " .. self.player.lives .. "\n" .. 
		"Level: " .. self.level,
		LIVESBOX.pos.x, 48, GAME_W - 100, "left")
	
	love.graphics.setColor(255, 255, 255, self:calculate_alpha(SCOREBOX))
	love.graphics.printf(
		"Score:____ " .. self.player.points .. "\n" ..
		"Highscore: " .. get_score(),
		48, 48, GAME_W - 100, "left")

	if self.current_wave.boss and next(self.clowns) then
		local barw = GAME_W - 64
		
		local hp = 0
		local maxhp = 0
		for i, v in ipairs(self.clowns) do
			 hp = hp + v.hp
			 maxhp = maxhp + creature_types[v.type].hp
		end
		
		local hpbar = hp / maxhp
		hpbar = hpbar * barw
		hpbar = math.ceil(hpbar/4) * 4
		
--		love.graphics.setColor(64, 2, 0, self:calculate_alpha(HEALTHBOX))
		love.graphics.setColor(64, 2, 0, 255)
		love.graphics.rectangle('fill', 32, 16, barw, 8)
--		love.graphics.setColor(213, 50, 45, self:calculate_alpha(HEALTHBOX))
		love.graphics.setColor(213, 22, 22, 255)
		love.graphics.rectangle('fill', 32, 16, hpbar, 8)
	end
	
	love.graphics.setColorMode("replace")
end

function game:calculate_alpha(box)
	local FADE_DIST = 80
	local MIN_ALPHA = 64
	local MAX_ALPHA = 255
	
	local alpha = max_distance(
			self.player.pos, self.player.size,
			box.pos, box.size) / FADE_DIST
	if alpha > 1 then alpha = 1 end
	
	return (MIN_ALPHA + alpha * (MAX_ALPHA - MIN_ALPHA))
end

function game:delete_obj(list, id)
	for i, v in ipairs(list) do
		if v.id == id then
			table.remove(list, i)
			break
		end
	end
end

-- pos is restricted to playing area
function game:fix_pos(c)
	if c.pos.x < 0 then
		c.pos.x = 0
	elseif c.pos.x + c.size.x >= GAME_W then
		c.pos.x = GAME_W - c.size.x
	end

	if c.pos.y < 0 then
		c.pos.y = 0
	elseif c.pos.y + c.size.y >= GAME_H then
		c.pos.y = GAME_H - c.size.y
	end
end

function game:out_of_area(c)
	if c.pos.x + c.size.x < 0 then
		return true
	elseif c.pos.x >= GAME_W then
		return true
	end

	if c.pos.y + c.size.y < 0 then
		return true
	elseif c.pos.y >= GAME_H then
		return true
	end

	return false
end

function game:update_player(dt)
	self.player.vel = vector(0, 0)
	
	if self.player.dead then
		self.player.death_anim:animate(dt)
		return
	end
	
	if keysdown[config.keys.left] then
		self.player.vel.x = self.player.vel.x - PLAYER_SPEED
	end
	if keysdown[config.keys.right] then
		self.player.vel.x = self.player.vel.x + PLAYER_SPEED
	end
	if keysdown[config.keys.up] then
		self.player.vel.y = self.player.vel.y - PLAYER_SPEED
	end
	if keysdown[config.keys.down] then
		self.player.vel.y = self.player.vel.y + PLAYER_SPEED
	end
	self.player.pos = self.player.pos + self.player.vel * dt

	self:fix_pos(self.player)
	
	if (love.mouse.isDown("l") or
		keysdown[config.keys.shoot_left] or
		keysdown[config.keys.shoot_right] or
		keysdown[config.keys.shoot_up] or
		keysdown[config.keys.shoot_down]) then
			
		local aim = vector(0, 0)

		if keysdown[config.keys.shoot_left] then
			aim.x = aim.x - 1
		end
		if keysdown[config.keys.shoot_right] then
			aim.x = aim.x + 1
		end
		if keysdown[config.keys.shoot_up] then
			aim.y = aim.y - 1
		end
		if keysdown[config.keys.shoot_down] then
			aim.y = aim.y + 1
		end

		if aim.x == 0 and aim.y == 0 then
			aim.x = love.mouse.getX() - self.player.pos.x - self.player.size.x/2
			aim.y = love.mouse.getY() - self.player.pos.y - self.player.size.y/2
		end
		
		if aim.x ~= 0 or aim.y ~= 0 then
			local time = self.timer:get_time()
			if self.player.last_shot + SHOOT_DELAY <= time then
				self.player.last_shot = time
				play_sound("laser.wav")
				self:add_shot(
					self.player.pos + self.player.size/2,
					aim,
					PLASMA_BALL,
					true)
			end
		end
	end
end

function game:spawn_enemies()
	if (not next(self.clowns) and self.wave_clear) then
		self.level = self.level + 1
		self.last_wave = self.timer:get_time()
		self.wave_clear = false
		self.current_wave =
			create_wave(
			9 + math.ceil(1.5 * self.level * self.level),
			self.level, 20, self.timer:get_time())
	end

	for _, cat in ipairs(self.current_wave) do
		if cat[2] >= 1 then
			local time = self.timer:get_time()
			local wave_dur = math.max(0, self.last_wave + cat[6] - cat[3])
			local spawn_delay = wave_dur / cat[2] + math.random() * 1.0
			local wait_time = time - cat[3]
				
			if wave_dur <= 0 then
				while cat[2] >= 1 do
					self:create_clown(cat[1], cat[4], cat[5])
					cat[3] = time
					cat[2] = cat[2] - 1
				end
			end
			
			if not next(self.clowns) and wait_time < spawn_delay then
				wait_time = wait_time + spawn_delay
			end

			for i = 1, math.floor(wait_time / spawn_delay) do
				self:create_clown(cat[1], cat[4], cat[5])
				
				cat[3] = time
				cat[2] = cat[2] - 1
				if cat[2] < 1 then
					break
				end
			end
		end
	end
	
	local clowns_left = 0
	for _, cat in ipairs(self.current_wave) do
		clowns_left = clowns_left + math.floor(cat[2])
	end
	if clowns_left < 1 then
		self.wave_clear = true
	end
end

function game:update(dt)
	self.timer:update(dt)
	self:update_player(dt)
	for i, v in ipairs(self.clowns) do
		if v.stays_inside then
			self:fix_pos(v)
		end
		v:update(dt)
	end

	for i, v in ipairs(self.player_shots) do
		v:update(dt)
	end

	for i, v in ipairs(self.enemy_shots) do
		v:update(dt)
	end
	
	if not self.player.dead then
		self:spawn_enemies()
	end

	self:check_collisions()

	if self.wave_clear then
		local only_mines_left = true
		for i, v in ipairs(self.clowns) do
			if v.type ~= MINE then
				only_mines_left = false
				break
			end
		end

		if only_mines_left then
			for i, v in ipairs(self.clowns) do
				if not v:dead() then
					self.timer:add_task(v.die, math.random() * 0.5, {v})
				end
			end
		end
	end
	
	if self.player.dead and self.current_wave.boss then
		local clowns_have_left = true
		for i, v in ipairs(self.clowns) do
			if not self:out_of_area(v) then
				clowns_have_left = false
				break
			end
		end

		if clowns_have_left then
			self.level = self.level - 1
			self:respawn()
		end
	end
end

function game:check_collisions()
	for _, c in ipairs(self.clowns) do
		if not c:dead() then
			if objs_collide(self.player, c) then
				self:hit_player()
				if c.type == MINE then
					c:die()
				end
			end

			for _, shot in ipairs(self.player_shots) do
				if objs_collide(shot, c) then
					c:hit(shot)
					self:delete_shot(shot.id)
				end
			end
		end
	end


	for _, shot in ipairs(self.enemy_shots) do
		if objs_collide(self.player, shot) then
			self:hit_player()
			if not self.player.dead then
				self:delete_shot(shot.id)
			end
		end
	end
end


function game:hit_player()
	if not self.player.dead then
		self.player.lives = self.player.lives - 1

		play_sound("putum.wav")

		self.player.dead = true
		self.player.death_anim = animation.new(explosion_anim_data)
		self.player.death_anim:set_mode("explode", false)
		
		if self.player.lives <= 0 then
			self.timer:add_task(self.die, 1, {self})
		else
			self.timer:add_task(self.clear_clowns, 0.8, {self})
		end
	end
end

function game:clear_clowns()
	self.enemy_shots = {}
	self.player_shots = {}

	for _, c in ipairs(self.clowns) do
		if not c:dead() then
			if not self.current_wave.boss then
				self.timer:add_task(c.die, math.random() * 0.5, {c})
			else
				self.timer:add_task(c.leave, math.random() * 0.5, {c})
			end
		end
	end
	
	if not self.current_wave.boss then
		self.timer:add_task(self.respawn, 1.05, {self})
	end
	
end

