require 'hump/gamestate'
require 'game'
require 'key_config'

menu = {}
menu.sel = 0

local titlecolor = {r=181; g=164; b=126; a=255}
local optcolor = {r=127; g=127; b=127; a=192}
local disabledcolor = {r=47; g=47; b=47; a=157}
local selcolor = {r=255; g=255; b=255; a=255}

local MENU_DIR = "menus/"

function menu:keypressed(key, uni)
	if key == "escape" then
		if self.menu.esc_action then
			self:perform_action(self.menu.esc_action)
		end
	elseif key == "up" then
		self:selectPrev()
	elseif key == "down" then
		self:selectNext()
	elseif key == "return" or key == "kpenter" then
		self:perform_action(self.sel)
	end
end

function menu:selectNext()
	self.sel = self.sel + 1
	if self.sel >= self.menu.size then
		self.sel = 0
	end
	while self.menu.options[self.sel].disabled do
		self.sel = self.sel + 1
		if self.sel >= self.menu.size then
			self.sel = 0
		end
	end
end

function menu:selectPrev()
	self.sel = self.sel - 1
	if self.sel < 0 then
		self.sel = self.menu.size - 1 
	end
	while self.menu.options[self.sel].disabled do
		self.sel = self.sel - 1
		if self.sel < 0 then
			self.sel = self.menu.size - 1 
		end
	end
end

function menu:keyreleased(key, uni)
end

function menu:perform_action(n)
	local action = self.menu.options[n].action
	if action == "q" then
		love.event.push("q")
	elseif action == "m" then
		if self.menu.options[n].back then
			self.back = self.menu.options[n].back
		end
		self:loadmenu(self.menu.options[n].menu)
	elseif action == "b" then
		if not self.back then
			self.back = "menu_main"
		end
		self:loadmenu(self.back)
	elseif action == "f" then
		self.menu.options[n].func()
	elseif action == "k" then
		Gamestate.switch(key_config, self.menu.options[n].key, 
			self.menu.options[n].key_name)
	elseif action == "game_restart" then
		game:restart_game()
		Gamestate.switch(game)
	elseif action == "game_resume" then
		Gamestate.switch(game)
	elseif action == "game_start" then
		game:restart(self.menu.options[n].level)
		Gamestate.switch(game)
	end
end

function menu:loadmenu(m, reload)
	if reload then
		self.menu = love.filesystem.load(MENU_DIR .. m .. ".lua")()
	else
		self.menu = require(MENU_DIR .. m)
	end
	self.sel = -1
	self:selectNext()
end

function menu:enter(prev, ...)
	love.graphics.setColorMode("modulate")
	if not self.menu then
		self:loadmenu("menu_main")
	end
	love.graphics.setBackgroundColor(0, 0, 0)
end

function menu:draw()
	local width = love.graphics.getWidth()
	local height = love.graphics.getHeight()
	local fonth = love.graphics.getFont():getHeight() + 10
	local x = (self.menu.x or 0) * width
	local y = (self.menu.y or 0.2) * height
	local maxy = (self.menu.maxy or 0.8) * height

	local function draw_option(text, color, i, noshadow)
		if not noshadow then
			love.graphics.setColor(0, 0, 0, 200)
			love.graphics.printf(text, x+2, y+i*fonth+2, width-x, self.menu.align or "center")
		end
		love.graphics.setColor(color.r, color.g, color.b, color.a)
		love.graphics.printf(text, x, y+i*fonth, width-x, self.menu.align or "center")
	end
	
	if self.menu.options[self.sel].bg then
		love.graphics.setColorMode("replace")
		love.graphics.draw(self.menu.options[self.sel].bg, 0, 0)
		love.graphics.setColorMode("modulate")
		love.graphics.setColor(0, 0, 0, 200)
		love.graphics.rectangle("fill", 0, 0, width, height)
	elseif self.menu.bg then
		love.graphics.setColorMode("replace")
		love.graphics.draw(get_image_raw(self.menu.bg), 0, 0)
		love.graphics.setColorMode("modulate")
	else
		love.graphics.setColorMode("replace")
		game:draw(true)
		love.graphics.setColorMode("modulate")
		love.graphics.setColor(0, 0, 0, 200)
		love.graphics.rectangle("fill", 0, 0, width, height)
	end

	if y+self.sel*fonth > maxy then
		y = maxy - self.sel*fonth
	end

	if self.menu.title then
		draw_option(self.menu.title, titlecolor, -2)
	end

	for i = 0, (self.menu.size - 1) do
		if i == self.sel then
			draw_option(self.menu.options[i].name, selcolor, i)
		elseif self.menu.options[i].disabled then
			draw_option(self.menu.options[i].name, disabledcolor, i, true)
		else
			draw_option(self.menu.options[i].name, optcolor, i)
		end
	end
end

function menu:update(dt)
end

function menu:leave() 
end

function menu:mousepressed()
end

function menu:mousereleased()
end

