require 'hump/vector'

shot = {}

PLASMA_BALL = 1
SLOW_PLASMA_BALL = 2
DROP_MINE = 3

local shot_types = {
	[PLASMA_BALL] = {
		img = "ball.png",
		img_pos = vector(8, 8),
		img_scale = 2,
		speed = 600,
		size = vector(32, 32),
		dmg = 2
	},
	[SLOW_PLASMA_BALL] = {
		img = "missile.png",
		img_pos = vector(-3, -3),
		img_scale = 1,
		speed = 300,
		size = vector(10, 10),
		dmg = 2
	}
}

function shot.new(pos, dir, type)
	local c = {
		pos = pos - shot_types[type].size/2,
		type = type,
		size = shot_types[type].size,
		vel = dir:normalized() * shot_types[type].speed,
		img = get_image_raw(shot_types[type].img),
		img_pos = shot_types[type].img_pos,
		img_scale = shot_types[type].img_scale,
		dmg = shot_types[type].dmg,
		id = -1
	}
	setmetatable(c, {__index = shot})
	return c
end

function shot:update(dt)
	self.pos = self.pos + self.vel * dt

	if game:out_of_area(self) then
		game:delete_shot(self.id)
	end
end

function shot:draw()
	local img = self.img
	love.graphics.draw(
		img,
		math.ceil(self.pos.x + self.img_pos.x),
		math.ceil(self.pos.y + self.img_pos.y),
		0, self.img_scale, self.img_scale)
end


