timer = {}

local t = {}
local mt = {
	__index = t
}

--- add a function to call after specified amount of time
-- Note that when adding a task in the game you should pass self in params 
-- since if you do add_task(function() self:do_stuff() end, 0) the timer may be
-- copied so the self refers to the old one and essentially breaks the thing
--
-- @param f The function to call
-- @param tt The time in seconds before the function is called
-- @params params The parameters passed to f as a table (they are unpack'd)
function t:add_task(f, tt, params)
	table.insert(self.tasks, {func = f, time = self.time + tt, params = params or {}})
end

function t:update(dt)
	self.time = self.time + dt
	for i = table.maxn(self.tasks), 1, -1 do
		if self.tasks[i].time < self.time then
			self.tasks[i].func(unpack(self.tasks[i].params))
			table.remove(self.tasks, i)
		end
	end
end

function t:deserialize()
	self.tasks = {}
end

function t:get_time()
	return self.time
end

function timer.create()
	local tm = {time = 0, tasks = {}}
	setmetatable(tm, mt)
	return tm
end

return timer
