require 'creature'

local waves = {

	[1] -- level number
	= {
		boss = false, -- boss level
		
		-- spawn groups
		{
			TROLL,	-- creature type
			
			5,	-- amount
			
			11,	-- spawn location:
				--      1   2   3
				--    .-----------.
				-- 12 |   GAME    | 4
				-- 11 |  WINDOW   | 5
				-- 10 |           | 6
				--    '-----------'
				--      9   8   7
				--
			
			0,	-- random variation in the spawn location
				-- always clockwise
				-- e.g. spawn loc 1 and variation 2 =
				-- creatures can spawn from anywhere above
			
			10	-- time in seconds, in which the creatures spawn
		},
	},

	[2] = { { TROLL, 10, 10, 2, 8 }, },
	[3] = { { TROLL, 6, 11, 0, 8 }, { TROLL, 6, 5, 0, 8 }, },
	[4] = { { TROLL, 10, 10, 2, 12 }, { TROLL, 10, 4, 2, 12 }, },
	[5] = { { TANK, 4, 11, 0, 4 }, },
	[6] = { { TANK, 4, 11, 0, 16 }, { TROLL, 10, 4, 2, 18 }, }, -- 5
	[7] = { { TANK, 2, 4, 2, 10 }, { TANK, 2, 4, 2, 10 }, { TROLL, 6, 9, 4, 16 }, },
	[8] = { { TROLL, 30, 8, 0, 3 }, },
	[9] = { { PLANE, 5, 10, 2, 8}, },
	[10] = { { TANK, 3, 2, 0, 8 }, { TROLL, 8, 5, 1, 16 }, { TROLL, 5, 11, 1, 8}, { PLANE, 4, 4, 2, 5} },
	[11] = { { TANK, 1, 12, 0, 1 }, { TANK, 1, 5, 0, 1 }, { TANK, 1, 6, 0, 1 }, { TANK, 1, 10, 0, 2 }, }, -- 10
	[12] = { { TANK, 6, 4, 2, 20 }, { TROLL, 10, 4, 2, 30 }, { MONK, 1, 12, 0, 4} },
	[13] = { { TANK, 5, 2, 0, 2 }, { TROLL, 5, 4, 0, 4 }, { TROLL, 5, 12, 0, 4} },
	[14] = { { TROLL, 4, 2, 0, 4}, { TROLL, 4, 8, 0, 4 }, { PLANE, 1, 11, 0, 2}, { PLANE, 1, 5, 0, 2} },
	[15] = { { TANK, 3, 3, 0, 6}, { TANK, 3, 9, 0, 6 }, { TROLL, 4, 8, 0, 2} },
	[16] = { { TROLL, 20, 1, 5, 10}, { BOMB_PLANE, 1, 12, 0, 1}, { BOMB_PLANE, 1, 5, 0, 1}, { BOMB_PLANE, 1, 10, 0, 1} },-- 15
	[17] = { { MONK, 1, 12, 0, 2}, { MONK, 1, 4, 0, 3}, { MONK, 1, 6, 0, 4}, { MONK, 1, 10, 0, 5 }, {TANK, 8, 8, 0, 8} },
	[18] = { { PLANE, 10, 10, 2, 20}, { BOMB_PLANE, 10, 4, 2, 10} },
	[19] = { { TANK, 3, 2, 0, 4}, { TANK, 3, 8, 0, 4}, { TROLL, 3, 11, 0, 8}, { TROLL, 3, 5, 0, 8} },
	[20] = { { PLANE, 20, 5, 0, 10}, { TANK, 8, 2, 0, 8} },
	[21] = { { TROLL, 8, 1, 0, 10}, { TROLL, 8, 7, 0, 10}, { BOMB_PLANE, 2, 11, 0, 8}, { BOMB_PLANE, 2, 5, 0, 8} }, -- 20
	[22] = { { TANK, 5, 12, 1, 14}, { TANK, 5, 6, 1, 14 }, { MONK, 2, 9, 1, 20}, { MONK, 2, 3, 1, 20} },
	[23] = { boss = true, { BOSS1, 1, 4, 0, 10} }
}

local function handmade_wave(total_str, level, max_size, time)
	if not waves[level] then
		return {}
	end
	
	local wave = {}
	
	for i, v in ipairs(waves[level]) do
		wave[i] = {
			v[1], v[2], time, v[3], v[4], v[5],
		}
	end

	wave.boss = waves[level].boss
	
	return wave
end

function get_procedural_start()
	return #waves
end

local function procedural_wave(total_str, level, max_size, time, block)
	local wave = {}

	block = block or {}
	
	local types = {}
	local amount = 0
	for k, v in pairs(creature_levels) do
		if k <= level then
			for i, t in ipairs(v) do
				table.insert(types, t)
				amount = amount + 1
			end
		end
	end

	if amount <= 0 then
		return wave
	end

	local creature_idx = types[math.random(amount)]

	local creature_type = creature_types[creature_idx]
	
	local spread = 1 + math.floor(math.min(level / creature_type.level * 3, 11)
			* math.random())
	local real_str = math.pow(((math.sqrt(spread) * 4 + 1.5 * (block[creature_idx] or 0)) * creature_type.strength), 1.4) / 50
	local size = total_str / real_str

	if size > max_size then
		size = max_size
	end
	
	size = size - math.ceil(size * (math.random() * 0.3 + 0.4))

	block[creature_idx] = (block[creature_idx] or 0) + size

	wave[1] = { creature_idx, size, time, math.random(1, 12), spread, 10 }

	-- we don't want planes moving vertically
	if creature_idx == PLANE or creature_idx == BOMB_PLANE then
		if wave[1][4] >= 1 and wave[1][4] <= 3 then
			wave[1][4] = wave[1][4] + 3
		elseif wave[1][4] >= 7 and wave[1][4] <= 9 then
			wave[1][4] = wave[1][4] + 3
		end
		if wave[1][4] >= 4 and wave[1][4] <= 6 then
			wave[1][5] = math.min(6 - wave[1][4], wave[1][5])
		end
		if wave[1][4] >= 10 and wave[1][4] <= 12 then
			wave[1][5] = math.min(12 - wave[1][4], wave[1][5])
		end
	end
	
	if size > 1 and size*real_str < total_str then
		local wave2 = procedural_wave(
			total_str - size*real_str,
			level,
			max_size,
			time,
			block)
		
		for i, v in ipairs(wave2) do
			table.insert(wave, v)
		end
	else
		block[creature_idx] = (block[creature_idx] or 0) - size
	end
	
	return wave
end

function create_wave(total_str, level, max_size, time)
	local ret = handmade_wave(total_str, level, max_size, time)
	if #ret ~= 0 then
		return ret
	end

	return procedural_wave(total_str, level, max_size, time)
end

--create_wave = handmade_wave
--create_wave = procedural_wave

